package sk.upjs.ics.rozhovor;

import sk.upjs.ics.rozhovor.dao.JdbcTemplateFactory;
import sk.upjs.ics.rozhovor.dao.MessageMapper;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;

public class DatabaseChat implements Chat {
    private static Logger LOG = Logger.getLogger(DatabaseChat.class.getName());

    private JdbcTemplate jdbcTemplate = JdbcTemplateFactory.getJdbcTemplate();

    private BeanPropertyRowMapper<User> userMapper = BeanPropertyRowMapper.newInstance(User.class);

    public void login(User user) throws LoginException {
        validujUsera(user);
        String sql = "INSERT INTO USER VALUES (NULL, ?)";

        jdbcTemplate.update(sql, user.getNick());
    }

    public void logout(User user) {
        String sql = "DELETE FROM USER WHERE nick = ?";
        jdbcTemplate.update(sql, user.getNick());
    }

    public void pridajSpravu(Message sprava) {
        int userId = najdiIdUsera(sprava.getUser());

        String sql = "INSERT INTO HISTORY VALUES(NULL, ?, ?, ?)";
        jdbcTemplate.update(sql,
                            sprava.getText(),
                            sprava.getTimestamp(),
                            userId
                            );
    }

    private int najdiIdUsera(User user) {
        String sql = "SELECT ID FROM USER WHERE nick = ?";
        return jdbcTemplate.queryForInt(sql, user.getNick());
    }

    private void validujUsera(User user) {
        String sql = "SELECT COUNT(*) FROM USER WHERE nick = ?";
        if(jdbcTemplate.queryForInt(sql, user.getNick()) > 0) {
            throw new LoginException("Pouzivatel " + user.getNick() + " je uz prihlaseny. Zvolte iny nick.");
        }
    }

    public List<Message> ziskajNoveSpravy(Date najstarsiaNovaSprava) {
        String sql = "SELECT * FROM HISTORY JOIN user on user.id = history.id_user WHERE \"TIMESTAMP\" > ? ORDER BY \"TIMESTAMP\"";
        List<Message> newMessages = jdbcTemplate.query(sql, new MessageMapper(), najstarsiaNovaSprava);
        LOG.fine("Ziskal som " + newMessages.size() + " novych sprav od " + najstarsiaNovaSprava);
        return newMessages;
    }

    public List<User> getOnlineUsers() {
        String sql = "SELECT * FROM user";
        return jdbcTemplate.query(sql, userMapper);
    }

    

    
}
