package sk.upjs.ics.rozhovor;

import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

public class MemoryChat implements Chat {
    private static final Logger logger = Logger.getLogger(MemoryChat.class.getName());

    private List<Message> historia = new LinkedList<Message>();

    private List<User> onlineUsers = new LinkedList<User>();

    public void pridajSpravu(Message sprava) {
        historia.add(sprava);

        logger.info("Nova sprava: " + sprava);
    }

    public void login(User user) throws LoginException {
        validujUsera(user);

        onlineUsers.add(user);

        logger.info("Prihlaseny user: " + user);
    }

    protected void validujUsera(User user) throws LoginException {
        if(onlineUsers.contains(user)) {
            throw new LoginException("Pouzivatel " + user + " je uz prihlaseny. Zvolte iny nick.");
        }
        if("hitler666".equals(user.getNick())) {
            throw new LoginException(
                    "Nick obsahuje meno, ktore nie je v sulade s pravidlami:" + user);
        }
    }

    public void logout(User user) {
        onlineUsers.remove(user);

        logger.info("User " + user + " sa odhlasil.");
    }

    public List<Message> ziskajNoveSpravy(Date najstarsiaNovaSprava) {
        List<Message> noveSpravy = new LinkedList<Message>();
        for (Message message : this.historia) {
            if(message.getTimestamp().after(najstarsiaNovaSprava)) {
                noveSpravy.add(message);
            }
        }
        return noveSpravy;
    }

    public List<User> getOnlineUsers() {
        return Collections.unmodifiableList(onlineUsers);
    }


    
}
