package sk.upjs.ics.rozhovor.gui;

import java.util.Date;
import java.util.List;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import javax.swing.JTextArea;
import javax.swing.SwingWorker;
import sk.upjs.ics.rozhovor.Chat;
import sk.upjs.ics.rozhovor.Message;

public class MessageRetrievalWorker extends SwingWorker<Void, Message> {

    /**
     * Textove pole, kam sa budu vypisovat spravy.
     */
    private JTextArea messagesTextArea;
    /**
     * Chat, z ktoreho sa budu periodicky vytahovat spravy.
     */
    private final Chat chat;

    public MessageRetrievalWorker(JTextArea messagesTextArea, Chat chat) {
        this.messagesTextArea = messagesTextArea;
        this.chat = chat;
    }

    /*
     * Timestamp poslednej spravy, ktoru sme vytiahli z historie v predoslom behu.
     */
    private Date timestampPoslednejZiskanejSpravy = new Date();

    @Override
    protected Void doInBackground() {
        while (true) {
            System.out.println("cycle");
            try {
                List<Message> newMessages = retrieveNewMessages();
                // JTextArea nemoze byt upravovane v tejto metode,
                // lebo ta NEbezi v EDT. Publishom posleme spravy
                // do process(), ktory bezi v EDT a teda v nom
                // mozeme upravovat swingovske komponenty
                publish(newMessages.toArray(new Message[0]));

                TimeUnit.MILLISECONDS.sleep(500);
            } catch (InterruptedException e) {
                System.out.println("Interrupt");
                return null;
            } catch (Throwable e) {
                //kazdu inu vynimku prebalime do runtime exception
                // a vyzdvihneme si ju v done();
                e.printStackTrace();
                throw new RuntimeException(e);
            }

        }
    }

    @Override
    protected void process(List<Message> chunks) {
        for (Message message : chunks) {
            messagesTextArea.setText(messagesTextArea.getText() + "\n" + message);
        }
    }

    private List<Message> retrieveNewMessages() {
        List<Message> spravy = chat.ziskajNoveSpravy(timestampPoslednejZiskanejSpravy);
        if(!spravy.isEmpty()) {
            Message poslednaSprava = spravy.get(spravy.size() - 1);
            timestampPoslednejZiskanejSpravy = poslednaSprava.getTimestamp();
        }
        return spravy;
    }

    @Override
    protected void done() {
        try {
            get();
        } catch (InterruptedException e) {
            throw new RuntimeException("Beh ulohy bol preruseny.", e);
        } catch (CancellationException e) {
            // uloha bola zrusena, nerobime nic
        } catch (ExecutionException e) {
            //tu ziskame vynimku vlastnu vynimku vyhodenu v doInBackground()
            throw new RuntimeException("Chyba pri vykonavani ulohy.", e);
        }
    }

}
