package sk.upjs.ics.rozhovor.gui;

import java.util.List;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.logging.Logger;
import javax.swing.JList;
import javax.swing.SwingWorker;
import sk.upjs.ics.rozhovor.Chat;
import sk.upjs.ics.rozhovor.User;

public class OnlineUserWorker extends SwingWorker<Void, List<User>> {
    private static final Logger logger = Logger.getLogger(OnlineUserWorker.class.getName());

    private Chat chat;
    
    private final JList userList;

    public OnlineUserWorker(Chat chat, JList userList) {
        this.chat = chat;
        this.userList = userList;
    }

    @Override
    protected Void doInBackground() throws Exception {
        while (!isCancelled()) {
            try {
                List<User> onlineUsers = chat.getOnlineUsers();
                // JTextArea nemoze byt upravovane v tejto metode,
                // lebo ta NEbezi v EDT. Publishom posleme pouzivatelov
                // do process(), ktory bezi v EDT a teda v nom
                // mozeme upravovat swingovske komponenty
                publish(onlineUsers);

                try {
                    TimeUnit.MILLISECONDS.sleep(500);
                } catch (InterruptedException e) {
                    // nerobime nic, vlakno moze byt zastavene cez cancel()
                }
            } catch (Throwable e) {
                logger.throwing(OnlineUserWorker.class.getName(), "doInBackground", e);
                //kazdu inu vynimku prebalime do runtime exception
                // a vyzdvihneme si ju v done();
                e.printStackTrace();
                throw new RuntimeException(e);
            }
        }
        return null;
    }

    @Override
    protected void process(List<List<User>> chunks) {
        if(isDone()) {
            logger.fine("Trying to update model when worker thread is done.");
            return;
        }
        logger.fine("Processing user list and updating GUI.");
        if(chunks.isEmpty()) {
            return;
        }
        List<User> mostRecentUserList = chunks.get(chunks.size() - 1);
        userList.setModel(new UserListModel(mostRecentUserList));
    }

    @Override
    protected void done() {
        try {
            logger.fine("Terminating online user worker.");
            userList.setModel(UserListModel.EMPTY_MODEL);
            logger.fine("User model set to empty list.");
            get();
        } catch (InterruptedException e) {
            throw new RuntimeException("Beh ulohy bol preruseny.", e);
        } catch (CancellationException e) {
            // uloha bola zrusena, nerobime nic
        } catch (ExecutionException e) {
            //tu ziskame vynimku vlastnu vynimku vyhodenu v doInBackground()
            throw new RuntimeException("Chyba pri vykonavani ulohy.", e);
        }
    }



}
